use ed25519_dalek::Signature;
use ed25519_dalek::Signer;
use ed25519_dalek::SigningKey;

pub struct ChallengeSigner {
    signing_key: SigningKey,
}

impl ChallengeSigner {
    pub fn generate_sign_key() -> [u8; 32] {
        let signing_key = SigningKey::generate(&mut rand::thread_rng());
        signing_key.to_bytes()
    }

    pub fn new() -> Self {
        let signing_key = SigningKey::generate(&mut rand::thread_rng());
        ChallengeSigner { signing_key }
    }

    pub fn new_from_bytes(bytes: [u8; 32]) -> Self {
        let signing_key = SigningKey::from_bytes(&bytes);
        ChallengeSigner { signing_key }
    }

    pub fn sign(&self, message: &[u8]) -> Signature {
        self.signing_key.sign(message)
    }

    pub fn verify(&self, message: &[u8], signature: &[u8; 64]) -> bool {
        let signature_result = Signature::from_bytes(signature);
        self.signing_key.verify(message, &signature_result).is_ok()
    }

}
